# -*-perl-*-
#
# $Id: algs.PL,v 1.2 2004/04/08 01:36:21 mdw Exp $
#
# Create tables of algorithms
#
# (c) 2001 Straylight/Edgeware
#

#----- Licensing notice -----------------------------------------------------
#
# This file is part of the Perl interface to Catacomb.
#
# Catacomb/Perl is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# Catacomb/Perl is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Catacomb/Perl; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

@cipher = qw(des desx des3
	     blowfish twofish
	     cast128 cast256
	     idea safer safersk
	     rc2 rc5
	     square rijndael rijndael192 rijndael256
	     serpent
	     skipjack
	     mars
	     tea xtea);
@stream = qw(rc4 seal);
@hash = qw(md5 md4 md2 tiger
	   sha sha256 sha384 sha512 
	   rmd128 rmd160 rmd256 rmd320);

sub enum {
  $x = shift;
  if (!ref($x)) { return $x; }
  elsif (ref($x) eq ARRAY) { return @$x }
  else { die "bad ref"; }
}

sub cross {
  my $x = [];
  foreach my $i (@_) {
    my @y = enum($i);
    if (@$x) {
      my @x = ();
      foreach my $j (@$x) { foreach my $k (@y) { push(@x, $j.$k); } }
      $x = \@x;
    } else {
      $x = \@y;
    }
  }
  return @$x;
}

open OUT, "> $ARGV[0]" or die "couldn't write `$ARGV[0].c': $!";

print OUT <<EOF;
/* -*-c-*-
 *
 * algs.c [generated]
 */

#include "catacomb-perl.h"

#include <catacomb/crc32.h>

EOF
print OUT cross("#include <catacomb/", \@cipher, "-",
		[qw(ecb cbc cfb counter ofb)], ".h>\n"), "\n";
print OUT cross("#include <catacomb/", \@stream, ".h>\n"), "\n";
print OUT cross("#include <catacomb/", \@hash, ".h>\n"), "\n";
print OUT cross("#include <catacomb/", \@hash, "-",
		[qw(mgf hmac)], ".h>\n"), "\n";

print OUT <<EOF;

const gccipher *ciphertab[] = {
EOF
print OUT cross("  &", \@cipher, "_", [qw(ecb cbc cfb counter ofb)], ",\n");
print OUT cross("  &", \@hash, "_", [qw(mgf)], ",\n");
print OUT cross("  &", \@stream, ",\n");
print OUT <<EOF;
  0
};

const gchash *hashtab[] = {
EOF
print OUT cross("  &", \@hash, ",\n");
print OUT <<EOF;
  &gcrc32,
  0
};

const gcmac *mactab[] = {
EOF
print OUT cross("  &", \@hash, "_", [qw(hmac nmac sslmac)], ",\n");
print OUT <<EOF;
  0
};

const struct randtab mgftab[] = {
EOF
foreach my $i (@hash) { print OUT "  { \"$i\", ${i}_mgfrand },\n"; }
print OUT <<EOF;
  { 0, 0 }
};

const struct randtab ctrtab[] = {
EOF
foreach my $i (@cipher) { print OUT "  { \"$i\", ${i}_counterrand },\n"; }
print OUT <<EOF;
  { 0, 0 }
};

const struct randtab ofbtab[] = {
EOF
foreach my $i (@cipher) { print OUT "  { \"$i\", ${i}_ofbrand },\n"; }
print OUT <<EOF;
  { 0, 0 }
};
EOF

#----- That's all, folks ----------------------------------------------------
