/* -*-c-*-
 *
 * $Id: catacomb-perl.h,v 1.2 2004/04/08 01:36:21 mdw Exp $
 *
 * Main header file for Catacomb/Perl
 *
 * (c) 2001 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of the Perl interface to Catacomb.
 *
 * Catacomb/Perl is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Catacomb/Perl is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Catacomb/Perl; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef CATACOMB_PERL_H
#define CATACOMB_PERL_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#include <assert.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#include <EXTERN.h>
#include <perl.h>
#include <XSUB.h>

#include <catacomb/gcipher.h>
#include <catacomb/ghash.h>
#include <catacomb/gmac.h>

#include <catacomb/grand.h>
#include <catacomb/fibrand.h>
#include <catacomb/lcrand.h>
#include <catacomb/dsarand.h>
#include <catacomb/rand.h>
#include <catacomb/noise.h>

#include <catacomb/passphrase.h>

#include <catacomb/mp.h>
#include <catacomb/mpint.h>
#include <catacomb/mpmul.h>
#include <catacomb/mprand.h>
#include <catacomb/mpcrt.h>
#include <catacomb/mpmont.h>
#include <catacomb/mpbarrett.h>

#include <catacomb/pfilt.h>
#include <catacomb/rabin.h>
#include <catacomb/pgen.h>
#include <catacomb/limlee.h>
#include <catacomb/strongprime.h>

/*----- Misc support ------------------------------------------------------*/

struct consttab { const char *name; UV val; };

extern U32 findconst(const struct consttab *cc,
		     const char *pkg, const char *name);

#define SET(sv, ob, ty) sv_setref_pv((sv), (char *)(ty), (void *)(ob))
#define MAKE(ob, ty) SET(NEWSV(0, 0), ob, ty)
#define RET(ob, ty) SET(sv_newmortal(), ob, ty)

/*----- Crypto algorithms -------------------------------------------------*/

struct randtab { const char *name; grand *(*rand)(const void *, size_t); };

typedef const octet keysize;
typedef gmac gMAC;
typedef gcmac gcMAC;

typedef grand Rand_True, Rand_DSA;

extern const gccipher *ciphertab[];
extern const gchash *hashtab[];
extern const gcmac *mactab[];
extern const struct randtab mgftab[], ctrtab[], ofbtab[];

extern SV *findrand(const struct randtab *rt, const char *cls,
		    const char *name, SV *k);
extern void listrand(const struct randtab *rt);

/*------ Multiprecision maths ---------------------------------------------*/

typedef mpmont MP_Mont;
typedef mpbarrett MP_Barrett;
typedef mpcrt MP_CRT;

#define XSINTERFACE_FUNC_SETMP(cv, f) \
  CvXSUBANY(cv).any_dptr = (void (*) _((void *)))(mp_##f)

#define SET_MP(sv, x) SET(sv, x, "Catacomb::MP")
#define RET_MP(x) RET(x, "Catacomb::MP")

extern mp *mp_fromiv(mp *d, IV iv);
extern IV mp_toiv(mp *x);
extern mp *mp_readsv(mp *m, SV *sv, STRLEN *off, int radix);
extern int mp_writesv(mp *m, SV *sv, int radix);
extern mp *mp_fromsv(SV *sv, const char *what, int radix, int keep, ...);

/*----- Prime generation --------------------------------------------------*/

typedef struct { pfilt pf; int rc; } MP_Prime_Filter;
typedef rabin MP_Prime_Rabin;
typedef SV MP_Prime_Gen_Proc, MP_Prime_Gen_NullProc;
typedef struct { pgen_proc *p; void *ctx; } pgmagic, MP_Prime_Gen_MagicProc;
typedef struct { pgmagic mg; pgen_filterctx f; } MP_Prime_Gen_FilterStepper;
typedef struct { pgmagic mg; pgen_jumpctx j; pfilt pf; }
  MP_Prime_Gen_JumpStepper;
typedef struct { pgmagic mg; rabin r; } MP_Prime_Gen_RabinTester;
typedef struct pgen_event MP_Prime_Gen_Event;

extern void pgproc_get(SV *sv, pgen_proc **p, void **ctx);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
