/* -*-c-*-
 *
 * $Id: pgproc.c,v 1.2 2004/04/08 01:36:21 mdw Exp $
 *
 * Prime generation procedures
 *
 * (c) 2001 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of the Perl interface to Catacomb.
 *
 * Catacomb/Perl is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Catacomb/Perl is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Catacomb/Perl; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*----- Header files ------------------------------------------------------*/

#include "catacomb-perl.h"

/*----- Main code ---------------------------------------------------------*/

static int perlevent(int rq, pgen_event *e, void *p)
{
  char *meth = 0;
  int n;
  SV *sv = p;
  int rc;
  dSP;

  switch (rq) {
    case PGEN_BEGIN:	meth = "pgen_begin"; break;
    case PGEN_TRY:	meth = "pgen_try"; break;
    case PGEN_FAIL:	meth = "pgen_fail"; break;
    case PGEN_PASS:	meth = "pgen_pass"; break;
    case PGEN_DONE:	meth = "pgen_done"; break;
    case PGEN_ABORT:	meth = "pgen_abort"; break;
    default:
      abort();
  }

  ENTER;
  SAVETMPS;
  PUSHMARK(SP);
  XPUSHs(sv);
  XPUSHs(sv_setref_pv(sv_newmortal(), "Catacomb::MP::Prime::Gen::Event",
		      (void *)e));
  PUTBACK;
  n = perl_call_method(meth, G_SCALAR);
  assert(n == 1);
  SPAGAIN;
  rc = POPi;
  PUTBACK;
  FREETMPS;
  LEAVE;
  return (rc);
}

void pgproc_get(SV *sv, pgen_proc **p, void **ctx)
{
  if (!SvOK(sv)) {
    *p = 0;
    *ctx = 0;
  } else if (sv_derived_from(sv, "Catacomb::MP::Prime::Gen::MagicProc")) {
    MP_Prime_Gen_MagicProc *mg =
      (MP_Prime_Gen_MagicProc *)SvIV((SV *)SvRV(sv));
    *p = mg->p;
    *ctx = mg->ctx;
  } else {
    *p = perlevent;
    *ctx = sv;
  }
}

/*----- That's all, folks -------------------------------------------------*/
